************************************************************************
* File (nc-qcd_ha_1414_1616.f) created on Fri Jul 18 11:11:46 MSD 2008.
************************************************************************
* This is the FORTRAN module (nc_qcd_ha_1414_1616) to calculate differential
* hard gluon Bremsstrahlung for dn + dn -> mo + mo process.
*
* It is produced by s2n package of SANC (v1.10) Project.
* Copyright (C), SANC Project Team, 2002-2008.
*
* http://sanc.jinr.ru/
* http://pcphsanc.cern.ch/
* E-mail: <sanc@jinr.ru>
************************************************************************
      subroutine nc_qcd_ha_1414_1616 (s,spr,ph4,costh4,costh5,hard)
      implicit none!
      include 's2n_declare.h'

      real*8 kappaz,sq,sqspr
      real*8 a1z,a2z,v1z,v2z,vaz
      real*8 p1p2,p1p3,p1p4,p1p5,p2p3,p2p4,p2p5,p3p4,p3p5,p4p5
      real*8 ph4,cosph4,sinph4,costh4,costh5,sinth4,sinth5
      complex*16 chizs,chizsc,chizspr,chizsprc

      real*8 betasprmomo,ibetasprmomo,betasdndn,ibetasdndn
      real*8 iz1dn,iz2dn
      real*8 v0spr,a0spr,vaspr

      sq = dsqrt(s)
      sqspr = dsqrt(spr)

      cosph4 = dcos(ph4)
      sinph4 = dsin(ph4)
      sinth4 = dsqrt(1d0-costh4**2)
      sinth5 = dsqrt(1d0-costh5**2)

      kappaz   = gf*rzm2/(dsqrt(2d0)*8d0*pi)/alpha
      chizs    = 4d0*kappaz*s/(s-mz2)
      chizsc   = dconjg(chizs)
      chizspr  = 4d0*kappaz*spr/(spr-mz2)
      chizsprc = dconjg(chizspr)

      a1z = adn*amo
      a2z = 4d0*vdn*adn*vmo*amo
      v1z = vdn*vmo
      v2z = (vdn**2+adn**2)*(vmo**2+amo**2)
      vaz = (vdn**2+adn**2)*(vmo**2-amo**2)

      betasprmomo = sqrt(1d0-4d0*rmom2/spr)
      ibetasprmomo = 1d0/betasprmomo
      betasdndn = sqrt(1d0-4d0*rdnm2/s)
      ibetasdndn = 1d0/betasdndn
      
      v0spr =
     & +qdn**2*qmo**2
     & +qdn*qmo*(chizspr+chizsprc)*v1z
     & +chizspr*chizsprc*v2z
      
      a0spr =
     & +qdn*qmo*(chizspr+chizsprc)*a1z
     & +chizspr*chizsprc*a2z
      
      vaspr =
     & +qdn**2*qmo**2
     & +qdn*qmo*(chizspr+chizsprc)*v1z
     & +chizspr*chizsprc*vaz

      p1p2=
     & -1d0/2*s

      p1p3=
     & +sinph4*sinth4*sinth5*(-1d0/4*betasprmomo*sq*sqspr)
     & +costh4*costh5*(1d0/8*betasprmomo*s+1d0/8*betasprmomo*spr)
     & +costh4*(-1d0/8*betasprmomo*s+1d0/8*betasprmomo*spr)
     & +costh5*(1d0/8*s-1d0/8*spr)
     & -1d0/8*s-1d0/8*spr

      p1p4=
     & +sinph4*sinth4*sinth5*(1d0/4*betasprmomo*sq*sqspr)
     & +costh4*costh5*(-1d0/8*betasprmomo*s-1d0/8*betasprmomo*spr)
     & +costh4*(1d0/8*betasprmomo*s-1d0/8*betasprmomo*spr)
     & +costh5*(1d0/8*s-1d0/8*spr)
     & -1d0/8*s-1d0/8*spr

      p1p5=
     & +costh5*(-1d0/4*s+1d0/4*spr)
     & -1d0/4*s+1d0/4*spr

      p2p3=
     & +sinph4*sinth4*sinth5*(1d0/4*betasprmomo*sq*sqspr)
     & +costh4*costh5*(-1d0/8*betasprmomo*s-1d0/8*betasprmomo*spr)
     & +costh4*(-1d0/8*betasprmomo*s+1d0/8*betasprmomo*spr)
     & +costh5*(-1d0/8*s+1d0/8*spr)
     & -1d0/8*s-1d0/8*spr

      p2p4=
     & +sinph4*sinth4*sinth5*(-1d0/4*betasprmomo*sq*sqspr)
     & +costh4*costh5*(1d0/8*betasprmomo*s+1d0/8*betasprmomo*spr)
     & +costh4*(1d0/8*betasprmomo*s-1d0/8*betasprmomo*spr)
     & +costh5*(-1d0/8*s+1d0/8*spr)
     & -1d0/8*s-1d0/8*spr

      p2p5=
     & +costh5*(1d0/4*s-1d0/4*spr)
     & -1d0/4*s+1d0/4*spr

      p3p4=
     & +mmo**2-1d0/2*spr

      p3p5=
     & +costh4*(-1d0/4*betasprmomo*s+1d0/4*betasprmomo*spr)
     & -1d0/4*s+1d0/4*spr

      p4p5=
     & +costh4*(1d0/4*betasprmomo*s-1d0/4*betasprmomo*spr)
     & -1d0/4*s+1d0/4*spr

      iz1dn=
     & +1d0/(mdn**2/s+1d0/4*betasdndn**2*sinth5**2)/(s-spr)*(1d0/2-1d0/2
     & *betasdndn*costh5)

      iz2dn=
     & +1d0/(mdn**2/s+1d0/4*betasdndn**2*sinth5**2)/(s-spr)*(1d0/2+1d0/2
     & *betasdndn*costh5)

      hard=
     & +v0spr*iz1dn**2*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(-1d0/3
     & *p1p3*p2p4*mdn**2/s**2/spr**2-1d0/3*p1p4*p2p3*mdn**2/s**2/spr**2+
     & 1d0/3*p2p3*p4p5*mdn**2/s**2/spr**2+1d0/3*p2p4*p3p5*mdn**2/s**2/sp
     & r**2)
     & +v0spr*iz1dn*iz2dn*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(1d0
     & /3*p1p3*p2p4/s/spr**2-1d0/6*p1p3*p4p5/s/spr**2+1d0/3*p1p4*p2p3/s/
     & spr**2-1d0/6*p1p4*p3p5/s/spr**2-1d0/6*p2p3*p4p5/s/spr**2-1d0/6*p2
     & p4*p3p5/s/spr**2)
     & +v0spr*iz1dn*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(1d0/3*p1p
     & 3*p1p4/s**2/spr**2-1d0/6*p1p3*p2p4/s**2/spr**2-1d0/6*p1p4*p2p3/s*
     & *2/spr**2+1d0/6*p2p3*p4p5/s**2/spr**2+1d0/6*p2p4*p3p5/s**2/spr**2
     & )
     & +v0spr*iz2dn**2*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(-1d0/3
     & *p1p3*p2p4*mdn**2/s**2/spr**2+1d0/3*p1p3*p4p5*mdn**2/s**2/spr**2-
     & 1d0/3*p1p4*p2p3*mdn**2/s**2/spr**2+1d0/3*p1p4*p3p5*mdn**2/s**2/sp
     & r**2)
     & +v0spr*iz2dn*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(-1d0/6*p1
     & p3*p2p4/s**2/spr**2+1d0/6*p1p3*p4p5/s**2/spr**2-1d0/6*p1p4*p2p3/s
     & **2/spr**2+1d0/6*p1p4*p3p5/s**2/spr**2+1d0/3*p2p3*p2p4/s**2/spr**
     & 2)
     & +a0spr*iz1dn**2*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(-1d0/3
     & *p1p3*p2p4*mdn**2/s**2/spr**2+1d0/3*p1p4*p2p3*mdn**2/s**2/spr**2-
     & 1d0/3*p2p3*p4p5*mdn**2/s**2/spr**2+1d0/3*p2p4*p3p5*mdn**2/s**2/sp
     & r**2)
     & +a0spr*iz1dn*iz2dn*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(1d0
     & /3*p1p3*p2p4/s/spr**2-1d0/6*p1p3*p4p5/s/spr**2-1d0/3*p1p4*p2p3/s/
     & spr**2+1d0/6*p1p4*p3p5/s/spr**2+1d0/6*p2p3*p4p5/s/spr**2-1d0/6*p2
     & p4*p3p5/s/spr**2)
     & +a0spr*iz1dn*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(-1d0/6*p1
     & p3*p2p4/s**2/spr**2+1d0/6*p1p4*p2p3/s**2/spr**2-1d0/6*p2p3*p4p5/s
     & **2/spr**2+1d0/6*p2p4*p3p5/s**2/spr**2)
     & +a0spr*iz2dn**2*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(-1d0/3
     & *p1p3*p2p4*mdn**2/s**2/spr**2+1d0/3*p1p3*p4p5*mdn**2/s**2/spr**2+
     & 1d0/3*p1p4*p2p3*mdn**2/s**2/spr**2-1d0/3*p1p4*p3p5*mdn**2/s**2/sp
     & r**2)
     & +a0spr*iz2dn*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(-1d0/6*p1
     & p3*p2p4/s**2/spr**2+1d0/6*p1p3*p4p5/s**2/spr**2+1d0/6*p1p4*p2p3/s
     & **2/spr**2-1d0/6*p1p4*p3p5/s**2/spr**2)
     & +vaspr*iz1dn**2*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(-1d0/6
     & *mdn**2*mmo**2/s**2/spr)
     & +vaspr*iz1dn*iz2dn*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(1d0
     & /6*mmo**2/spr**2)
     & +vaspr*iz1dn*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(-1d0/12*m
     & mo**2/s**2/spr-1d0/12*mmo**2/s/spr**2)
     & +vaspr*iz2dn**2*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(-1d0/6
     & *mdn**2*mmo**2/s**2/spr)
     & +vaspr*iz2dn*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(-1d0/12*m
     & mo**2/s**2/spr-1d0/12*mmo**2/s/spr**2)
     & +vaspr*betasprmomo/pi*alpha**2*alphas*(s-spr)*cf*(-1d0/6*mmo**2/s
     & **2/spr**2)

      hard = hard*conhc

      return
      end
